/*
 * Mike Glover
 * mpg4@duluoz.net
 *
 * pam_sql
 * PAM module for authenticating to an SQL database
 *
 * Loosely based on pam_mysql by:
 *   Gunay ARSLAN <arslan@gunes.medyatext.com.tr>
 *   James O'Kane <jo2y@midnightlinux.com>
 */

#define PAM_SM_ACCOUNT

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <security/pam_modules.h>

#include "db.h"
#include "misc.h"
#include "options.h"
#include "pam_sql.h"



PAM_EXTERN int pam_sm_acct_mgmt (pam_handle_t *pamh, int flags,
				 int argc, const char **argv)
{
  opt_t *opts;
  db_conn *conn;
  db_result *result;
  const char *user;
  char query[BUFLEN];
  int retval;

  
  /* load the options */
  retval = pam_get_data (pamh, _PAM_SQL_OPT_HANDLE, (const void **) &opts);
  if ( ( retval != PAM_SUCCESS ) || ( ! opts ) ) {
    opts = options_parse(argc, argv);
    if ( ! opts ) {
      syslog (LOG_ALERT, "Parse error. Exiting");
      return PAM_AUTHINFO_UNAVAIL;
    }
    pam_set_data (pamh, _PAM_SQL_OPT_HANDLE, (void *) opts, pam_sql_opt_free);
  }


  /* get the username */
  retval = pam_get_user (pamh, &user, NULL);
  if ( ( retval != PAM_SUCCESS ) || ( ! user ) ) {
    syslog (LOG_NOTICE, "Failed fetching username");
    return retval;
  }   
  

  /* connect to the database */
  conn = db_connect (opts);
  if ( ! conn ) {
    return PAM_AUTH_ERR;
  }

  /* write out the query */
  snprintf (query, BUFLEN-1, 
	    "select %s from %s where %s='%s' and %s>now()",
	    opts->usercol, opts->table, opts->usercol, user,
	    opts->expcol);

  /* execute the query */
  result = db_exec (conn, query);
  if ( ! result ) {
    syslog (LOG_ERR, "Query failed.");
    return PAM_AUTH_ERR;
  }

  /* should be exactly one row */
  if ( db_numrows(result) != 1 ) {
    syslog (LOG_NOTICE, "Account expired for user %s", user);
  }

  return PAM_SUCCESS;
}

