/*
 * Mike Glover
 * mpg4@duluoz.net
 *
 * pam_sql
 * PAM module for authenticating to an SQL database
 *
 * Loosely based on pam_mysql by:
 *   Gunay ARSLAN <arslan@gunes.medyatext.com.tr>
 *   James O'Kane <jo2y@midnightlinux.com>
 */

#define PAM_SM_AUTH

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <security/pam_modules.h>

#include "db.h"
#include "misc.h"
#include "options.h"
#include "pam_sql.h"


/* 
 * pam_sm_authenticate
 * get the username and password from the application
 * and check against the database
 */
PAM_EXTERN int pam_sm_authenticate (pam_handle_t * pamh, int flags,
				    int argc, const char **argv)
{
  int retval, i;
  const char *user, *passwd;
  char query[BUFLEN];
  db_conn *conn;
  db_result *result;
  opt_t *opts;

  passwd = NULL; /* ?? */

#ifdef HAVE_PAM_FAIL_DELAY
  pam_fail_delay (pamh, 2500);
#endif

  /* load options */
  retval = pam_get_data (pamh, _PAM_SQL_OPT_HANDLE, (const void **) &opts);
  if ( ( retval != PAM_SUCCESS ) || ( ! opts ) ) {
    opts = options_parse(argc, argv);
    if ( ! opts ) {
      syslog (LOG_ALERT, "Parse error. Exiting");
      return PAM_AUTHINFO_UNAVAIL;
    }
    pam_set_data (pamh, _PAM_SQL_OPT_HANDLE, (void *) opts, pam_sql_opt_free);
  }


  /* get the username */
  retval = pam_get_user (pamh, &user, NULL);
  if ( ( retval != PAM_SUCCESS ) || ( ! user ) ) {
    syslog (LOG_NOTICE, "Failed fetching username");
    return retval;
  } 


  /* talk to the user on the other end */
  /* FIXME: error check? */
  conversation(pamh);
  

  /* get the password */
  retval = pam_get_item (pamh, PAM_AUTHTOK, (const void **) &passwd);
  if ( retval != PAM_SUCCESS ) {
    syslog (LOG_NOTICE, "No authtoken provided: %s", 
	    pam_strerror(pamh, retval));
    return retval;
  }
  

  /* connect to the database */
  /* db_connect logs it's own errors */
  conn = db_connect (opts);
  if ( ! conn ) {
    return PAM_AUTHINFO_UNAVAIL;
  }
  
  
  /* set up the query string */
  snprintf (query, BUFLEN-1, 
	    "select %s from %s where %s='%s' and %s='%s'", 
	    opts->usercol, opts->table, opts->usercol, user, 
	    opts->passcol, passwd);

  result = db_exec (conn, query);
  if ( ! result ) {
    syslog (LOG_ERR, "Query failed");
    return PAM_AUTH_ERR;
  }

  if ( db_numrows(result) != 1 ) {
    syslog (LOG_NOTICE, "Authentication failed for user %s", user);
    return PAM_AUTH_ERR;
  }

  return PAM_SUCCESS;  
}



/* just a dummy function */
PAM_EXTERN int pam_sm_setcred (pam_handle_t *pamh, int flags, 
			       int argc, const char **argv)
{
  return PAM_SUCCESS;
}

